@@ -24,9 +24,9 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
-$Octasic_Revision: 56 $
+$Octasic_Revision: 57 $
 
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 
@@ -196,6 +196,8 @@ $Octasic_Revision: 56 $
 #define cOCT6100_TLV_TYPE_RESET_RIN_ALC_ON_TONES_BOFF		144
 #define cOCT6100_TLV_TYPE_RESET_SOUT_ALC_ON_TONES_BOFF		145
 
+#define cOCT6100_TLV_TYPE_RIN_ANR_VALUE						147
+
 #define cOCT6100_TLV_TYPE_ADPCM_ENABLE						150
 #define cOCT6100_TLV_TYPE_NUM_TONE_DETECTOR					151
 #define cOCT6100_TLV_TYPE_CONFERENCING_ENABLE				152
@@ -369,6 +371,9 @@ $Octasic_Revision: 56 $
 #define cOCT6100_TLV_MIN_LENGTH_RIN_ANR_BOFF_RW					8
 #define cOCT6100_TLV_MAX_LENGTH_RIN_ANR_BOFF_RW					8
 
+#define cOCT6100_TLV_MIN_LENGTH_RIN_ANR_VALUE_RW				8
+#define cOCT6100_TLV_MAX_LENGTH_RIN_ANR_VALUE_RW				8
+
 #define cOCT6100_TLV_MIN_LENGTH_RIN_MUTE_BOFF_RW				8
 #define cOCT6100_TLV_MAX_LENGTH_RIN_MUTE_BOFF_RW				8
 #define cOCT6100_TLV_MIN_LENGTH_SIN_MUTE_BOFF_RW				8
