@@ -1,8 +1,8 @@
 /*
  * Wildcard S100U USB FXS Interface Zapata Telephony Driver
  *
- * Written by Mark Spencer <markster@linux-support.net>
- *            Matthew Fredrickson <creslin@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
+ *            Matthew Fredrickson <creslin@digium.com>
  *
  * Copyright (C) 2001, Linux Support Services, Inc.
  *
@@ -184,7 +184,7 @@ static int Wcusb_ReadWcRegs(struct usb_device *dev, unsigned char index,
 }					  
 
 #ifdef USB2420
-#ifdef LINUX26
+#if defined(LINUX26) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
 static int wcusb_async_read(struct wc_usb_pvt *p, unsigned char index, unsigned char *data, int len, int state, void (*complete)(struct urb *urb, struct pt_regs *regs));
 static int wcusb_async_write(struct wc_usb_pvt *p, unsigned char index, unsigned char *data, int len, int state, void (*complete)(struct urb *urb, struct pt_regs *regs));
 static void wcusb_async_control(struct urb *urb, struct pt_regs *regs);
@@ -220,7 +220,7 @@ static void proslic_write_direct_async(struct wc_usb_pvt *p, unsigned char addre
 }
 
 #ifdef USB2420
-#ifdef LINUX26
+#if defined(LINUX26) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
 static void wcusb_async_control(struct urb *urb, struct pt_regs *regs)
 #else
 static void wcusb_async_control(struct urb *urb)
@@ -312,7 +312,7 @@ static void wcusb_async_control(urb_t *urb)
 }
 
 #ifdef USB2420
-#ifdef LINUX26
+#if defined(LINUX26) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
 static void keypad_check_done(struct urb *urb, struct pt_regs *regs)
 #else
 static void keypad_check_done(struct urb *urb)
@@ -429,7 +429,7 @@ static void wcusb_check_interrupt(struct wc_usb_pvt *p)
 }
 
 #ifdef USB2420
-#ifdef LINUX26
+#if defined(LINUX26) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
 static int wcusb_async_read(struct wc_usb_pvt *p, unsigned char index, unsigned char *data, int len, int state, void (*complete)(struct urb *urb, struct pt_regs *regs))
 #else
 static int wcusb_async_read(struct wc_usb_pvt *p, unsigned char index, unsigned char *data, int len, int state, void (*complete)(struct urb *urb))
@@ -477,7 +477,7 @@ static int wcusb_async_read(struct wc_usb_pvt *p, unsigned char index, unsigned
 }
 
 #ifdef USB2420
-#ifdef LINUX26
+#if defined(LINUX26) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
 static int wcusb_async_write(struct wc_usb_pvt *p, unsigned char index, unsigned char *data, int len, int state, void (*complete)(struct urb *urb, struct pt_regs *regs))
 #else
 static int wcusb_async_write(struct wc_usb_pvt *p, unsigned char index, unsigned char *data, int len, int state, void (*complete)(struct urb *urb))
@@ -883,7 +883,7 @@ static char wc_dtmf(struct wc_usb_pvt *p)
 	return ZT_LIN2MU(linsample);
 }
 
-#ifdef LINUX26
+#if defined(LINUX26)  && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
 static void wcusb_read_complete(struct urb *q, struct pt_regs *regs)
 #else
 static void wcusb_read_complete(struct urb *q)
@@ -949,7 +949,7 @@ static void wcusb_read_complete(struct urb *q)
 	return;
 }
 
-#ifdef LINUX26
+#if defined(LINUX26) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
 static void wcusb_write_complete(struct urb *q, struct pt_regs *regs)
 #else
 static void wcusb_write_complete(struct urb *q)
@@ -1476,7 +1476,7 @@ static void __exit wc_cleanup(void)
 	usb_deregister(&wc_usb_driver);
 }
 
-MODULE_AUTHOR("Matthew Fredrickson <creslin@linux-support.net>");
+MODULE_AUTHOR("Matthew Fredrickson <creslin@digium.com>");
 MODULE_DESCRIPTION("Wildcard USB FXS Interface driver");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
