@@ -31,11 +31,11 @@
 #include "zap_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_fxo.c 2243 2007-02-28 00:05:59Z tzafrir $";
+static const char rcsid[] = "$Id: card_fxo.c 2589 2007-06-07 21:54:07Z tzafrir $";
 
-DEF_PARM(int, print_dbg, 0, 0600, "Print DBG statements");
-DEF_PARM(uint, poll_battery_interval, 100, 0600, "Poll battery interval in milliseconds (0 - disable)");
-DEF_PARM(int, ring_debounce, 50, 0600, "Number of ticks to debounce a false RING indication");
+DEF_PARM(int, print_dbg, 0, 0644, "Print DBG statements");
+DEF_PARM(uint, poll_battery_interval, 500, 0644, "Poll battery interval in milliseconds (0 - disable)");
+DEF_PARM(int, ring_debounce, 50, 0644, "Number of ticks to debounce a false RING indication");
 
 /* Signaling is opposite (fxs signalling for fxo card) */
 #if 1
@@ -82,6 +82,9 @@ static int handle_register_command(xpd_t *xpd, char *cmdline);
 #define	DAA_CURRENT_REGISTER	0x1C
 #define	DAA_RING_REGISTER	0x05
 
+#define	POWER_DENIAL_CURRENT	3
+#define	POWER_DENIAL_TIME	1000	/* ticks */
+
 struct FXO_priv_data {
 	struct proc_dir_entry	*regfile;
 	struct proc_dir_entry	*fxo_info;
@@ -210,7 +213,7 @@ static int do_sethook(xpd_t *xpd, int pos, bool to_offhook)
 	xbus_t			*xbus;
 	struct FXO_priv_data	*priv;
 	int			ret = 0;
-	bool			value;
+	byte			value;
 
 	BUG_ON(!xpd);
 	BUG_ON(xpd->direction == TO_PHONE);		// We can SETHOOK state only on PSTN
@@ -684,6 +687,7 @@ static void update_battery_status(xpd_t *xpd, byte data_low, lineno_t chipsel)
 					BIT_CLR(priv->polarity, chipsel);
 				priv->polarity_counter[chipsel] = 0;
 				/* Inform Zaptel */
+				DBG("%s/%s/%d: Send ZT_EVENT_POLARITY\n", xpd->xbus->busname, xpd->xpdname, chipsel);
 				zt_qevent_lock(&xpd->chans[chipsel], ZT_EVENT_POLARITY);
 #if 0
 				/*
@@ -707,9 +711,11 @@ static void update_power_denial(xpd_t *xpd, byte data_low, lineno_t chipsel)
 
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	if (IS_SET(xpd->offhook, chipsel) && data_low < 3) {
+	if (IS_SET(xpd->offhook, chipsel) && data_low < POWER_DENIAL_CURRENT) {
+		/* Current dropped */
 		priv->current_counter[chipsel]++;
-		if (priv->current_counter[chipsel] >= 10) {
+		if (priv->current_counter[chipsel] * poll_battery_interval >= POWER_DENIAL_TIME) {
+			DBG("%s/%s/%d: Power Denial Hangup\n", xpd->xbus->busname, xpd->xpdname, chipsel);
 			priv->current_counter[chipsel] = 0;
 			do_sethook(xpd, chipsel, 0);
 			update_line_status(xpd, chipsel, 0);
@@ -833,6 +839,10 @@ static int proc_fxo_info_read(char *page, char **start, off_t off, int count, in
 	for_each_line(xpd, i) {
 		len += sprintf(page + len, "%2d ", IS_SET(priv->battery, i));
 	}
+	len += sprintf(page + len, "\n\t%-17s: ", "polarity");
+	for_each_line(xpd, i) {
+		len += sprintf(page + len, "%2d ", IS_SET(priv->polarity, i));
+	}
 	len += sprintf(page + len, "\n");
 	spin_unlock_irqrestore(&xpd->lock, flags);
 	if (len <= off+count)
