@@ -31,11 +31,11 @@
 #include "zap_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_fxs.c 2239 2007-02-27 06:14:18Z tzafrir $";
+static const char rcsid[] = "$Id: card_fxs.c 2475 2007-04-29 20:19:13Z tzafrir $";
 
-DEF_PARM(int, print_dbg, 0, 0600, "Print DBG statements");	/* must be before zap_debug.h */
-DEF_PARM_BOOL(poll_digital_inputs, 1, 0600, "Poll Digital Inputs");
-DEF_PARM_BOOL(reversepolarity, 0, 0600, "Reverse Line Polarity");
+DEF_PARM(int, print_dbg, 0, 0644, "Print DBG statements");	/* must be before zap_debug.h */
+DEF_PARM(uint, poll_digital_inputs, 1000, 0644, "Poll Digital Inputs");
+DEF_PARM_BOOL(reversepolarity, 0, 0644, "Reverse Line Polarity");
 
 /* Signaling is opposite (fxo signalling for fxs card) */
 #if 1
@@ -782,7 +782,6 @@ static void detect_vmwi(xpd_t *xpd)
 
 static int FXS_card_tick(xbus_t *xbus, xpd_t *xpd)
 {
-	static int		rate_limit = 0;
 	struct FXS_priv_data	*priv;
 
 	BUG_ON(!xpd);
@@ -790,9 +789,8 @@ static int FXS_card_tick(xbus_t *xbus, xpd_t *xpd)
 	BUG_ON(!priv);
 #if POLL_DIGITAL_INPUTS
 	if(poll_digital_inputs && xpd->id == 0) {
-		if((rate_limit++ % 1000) == 0) {
+		if((xpd->timer_count % poll_digital_inputs) == 0)
 			poll_inputs(xbus, xpd);
-		}
 	}
 #endif
 	handle_fxs_leds(xpd);
@@ -1335,8 +1333,8 @@ int __init card_fxs_startup(void)
 {
 	INFO("%s revision %s\n", THIS_MODULE->name, XPP_VERSION);
 #ifdef	POLL_DIGITAL_INPUTS
-	INFO("FEATURE: %s with DIGITAL INPUTS support (%s activated)\n",
-			THIS_MODULE->name, (poll_digital_inputs) ? "is" : "is not");
+	INFO("FEATURE: %s with DIGITAL INPUTS support (polled every %d msec)\n",
+			THIS_MODULE->name, poll_digital_inputs);
 #else
 	INFO("FEATURE: %s without DIGITAL INPUTS support\n", THIS_MODULE->name);
 #endif
