@@ -5,14 +5,17 @@ package Zaptel;
 # This program is free software; you can redistribute and/or
 # modify it under the same terms as Perl itself.
 #
-#use strict;
+# $Id: Zaptel.pm 2529 2007-05-17 19:34:32Z tzafrir $
+#
+use strict;
+use Zaptel::Span;
 
 my $proc_base = "/proc/zaptel";
 
 sub spans() {
 	my @spans;
 
-	-d $proc_base or die "Missing '$proc_base'. Perhaps zaptel module isn't loaded?\n";
+	-d $proc_base or return ();
 	foreach my $zfile (glob "$proc_base/*") {
 		$zfile =~ s:$proc_base/::;
 		my $span = Zaptel::Span->new($zfile);
