@@ -5,11 +5,14 @@ package Zaptel::Xpp::Xpd;
 # This program is free software; you can redistribute and/or
 # modify it under the same terms as Perl itself.
 #
-#use strict;
+# $Id: Xpd.pm 2529 2007-05-17 19:34:32Z tzafrir $
+#
+use strict;
 
 my $proc_base = "/proc/xpp";
 
 # Accessors (miniperl does not have Class:Accessor)
+our $AUTOLOAD;
 sub AUTOLOAD {
 	my $self = shift;
 	my $name = uc($AUTOLOAD);
@@ -21,6 +24,32 @@ sub AUTOLOAD {
 	}
 }
 
+sub blink($$) {
+	my $self = shift;
+	my $on = shift;
+	my $result;
+
+	my $file = "$proc_base/" . $self->fqn . "/blink";
+	die "$file is missing" unless -f $file;
+	# First query
+	open(F, "$file") or die "Failed to open $file for reading: $!";
+	$result = <F>;
+	chomp $result;
+	close F;
+	if(defined($on) and $on ne $result) {		# Now change
+		open(F, ">$file") or die "Failed to open $file for writing: $!";
+		print F ($on)?"1":"0";
+		if(!close(F)) {
+			if($! == 17) {	# EEXISTS
+				# good
+			} else {
+				undef $result;
+			}
+		}
+	}
+	return $result;
+}
+
 sub zt_registration($$) {
 	my $self = shift;
 	my $on = shift;
@@ -53,11 +82,17 @@ sub new($$) {
 	bless $self, $pack;
 	my $dir = "$proc_base/" . $self->fqn;
 	$self->{DIR} = $dir;
-	my ($name) = glob "$dir/*_info";
-	die "Missing info file in $dir" unless $name;
-	$name =~ s|^.*/||;	# basename
-	die "Bad info file name ($name) in $dir" if $name !~ /(\w+)_info/;
-	$self->{TYPE} = uc($1);
+	open(F, "$dir/summary") || die "Missing summary file in $dir";
+	my $head = <F>;
+	chomp $head;
+	# "XPD-00 (BRI_TE ,card present, span registered) SYNC MASTER"
+	close F;
+	$head =~ s/^.*\(//;
+	$head =~ s/\) */, /;
+	$head =~ s/\s*,\s*/,/g;
+	my ($type,$present,$registered,$sync) = split(/,/, $head);
+	$self->{TYPE} = uc($type);
+	$self->{IS_SYNC_MASTER} = ($sync =~ /MASTER/);
 	return $self;
 }
 
