@@ -43,11 +43,11 @@
 #endif
 #include "zap_debug.h"
 
-static const char rcsid[] = "$Id: xbus-core.c 2257 2007-03-01 19:29:12Z tzafrir $";
+static const char rcsid[] = "$Id: xbus-core.c 2529 2007-05-17 19:34:32Z tzafrir $";
 
 /* Defines */
 #define	POLL_TIMEOUT		(2*MAX_XPDS)	/* in jiffies */
-#define	INITIALIZATION_TIMEOUT	(40*HZ)		/* in jiffies */
+#define	INITIALIZATION_TIMEOUT	(60*HZ)		/* in jiffies */
 #define	PROC_XBUSES		"xbuses"
 #define	PROC_XBUS_SUMMARY	"summary"
 #define	PROC_XBUS_WAITFOR_XPDS	"waitfor_xpds"
@@ -59,7 +59,7 @@ static int proc_xbus_command_write(struct file *file, const char __user *buffer,
 
 /* Command line parameters */
 extern int print_dbg;
-DEF_PARM(uint, poll_timeout, POLL_TIMEOUT, 0600, "Timeout (in jiffies) waiting for units to reply (default " __stringify(POLL_TIMEOUT) ")");
+DEF_PARM(uint, poll_timeout, POLL_TIMEOUT, 0644, "Timeout (in jiffies) waiting for units to reply");
 
 /* Forward declarations */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
@@ -830,6 +830,8 @@ xbus_t *xbus_new(xbus_ops_t *ops)
 	INFO("New xbus: %s\n", xbus->busname);
 	init_waitqueue_head(&xbus->packet_cache_empty);
 	atomic_set(&xbus->packet_counter, 0);
+	xbus->min_tx_sync = INT_MAX;
+	xbus->min_rx_sync = INT_MAX;
 
 	xbus->num_xpds = 0;
 	poller = poller_new(xbus);
@@ -1008,6 +1010,12 @@ static int xbus_read_proc(char *page, char **start, off_t off, int count, int *e
 			xbus->open_counter,
 			atomic_read(&xbus->packet_counter)
 		      );
+	len += sprintf(page + len, "\nPLL DRIFT=%d\n", xbus->sync_adjustment);
+	len += sprintf(page + len, "PCM Metrices:\n");
+	len += sprintf(page + len, "\tPCM TX: min=%ld  max=%ld\n",
+				xbus->min_tx_sync, xbus->max_tx_sync);
+	len += sprintf(page + len, "\tPCM RX: min=%ld  max=%ld\n",
+				xbus->min_rx_sync, xbus->max_rx_sync);
 	len += sprintf(page + len, "COUNTERS:\n");
 	for(i = 0; i < XBUS_COUNTER_MAX; i++) {
 		len += sprintf(page + len, "\t%-15s = %d\n",
@@ -1271,6 +1279,7 @@ int __init xbus_core_init(void)
 	DBG("Creating debugfs xpp root\n");
 	debugfs_root = debugfs_create_dir("xpp", NULL);
 	if(!debugfs_root) {
+		ERR("Failed to create debugfs root\n");
 		ret = -EFAULT;
 		goto err;
 	}
