@@ -42,8 +42,9 @@
 #include "xbus-core.h"
 #include "xproto.h"
 #include "xpp_zap.h"
+#include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xpp_zap.c 2239 2007-02-27 06:14:18Z tzafrir $";
+static const char rcsid[] = "$Id: xpp_zap.c 2529 2007-05-17 19:34:32Z tzafrir $";
 
 #ifdef CONFIG_PROC_FS
 struct proc_dir_entry *xpp_proc_toplevel = NULL;
@@ -64,34 +65,38 @@ static unsigned int		xpp_timer_count = 0;
 static unsigned int		xpp_last_jiffies = 0;
 
 #ifdef	ZAPTEL_SYNC_TICK
+static unsigned int            zaptel_tick_count = 0;
 /*
  * Statistics variables
  */
 static struct timeval		ticked_zaptel;
 static struct timeval		ticked_xpp;
 static long			usec_lag_curr;	/* current: zaptel - xpp */
-static long			usec_lag_prev;	/* previous: zaptel - xpp */
-static long			usec_delta;	/* previous - current */
-static long			usec_sigma;	/* sum of deltas */
-static long			status_cleared_at;
-static long			since_status_clear;
+static long			sigma_lag;	/* sum of lags over SYNC_ADJ_INTERVAL */
+static long			average_lag;	/* average of lags over SYNC_ADJ_INTERVAL */
 static bool			zaptel_is_ticking;
+static bool			have_sync_mastership;
+static unsigned int		sync_tick_nomaster;
 
-#define	ZAPTEL_BIG_LAG		500 /* usec */
+#define	ZAPTEL_BIG_LAG		2000	/* usec */
 
-DEF_PARM_BOOL(sync_tick_active, 1, 0600, "Measure via zaptel sync_tick() method");
+#define	SYNC_ADJ_MIN            (-30)	/* minimal firmware drift unit */
+#define	SYNC_ADJ_MAX            30	/* maximal firmware drift unit */
+#define	SYNC_ADJ_INTERVAL       5000	/* minimum interval between fixes (usec) */
+#define	SYNC_ADJ_FACTOR         30	/* sigma usec/drift_unit */
+
+DEF_PARM_BOOL(sync_tick_active, 1, 0644, "Measure via zaptel sync_tick() method");
 
-static void sync_status_clear(void);
 #endif
 
 
-DEF_PARM_BOOL(pcm_tasklet, 0, 0600, "Handle PCM in a tasklet (lower interrupt load)");
-DEF_PARM(int, disable_pcm, 0, 0600, "Disable all PCM transmissions");
-DEF_PARM(int, print_dbg, 0, 0600, "Print DBG statements");
-DEF_PARM_BOOL(zap_autoreg, 1, 0600, "Register spans automatically (1) or not (0)");
-DEF_PARM_BOOL(prefmaster, 1, 0600, "Do we want to be zaptel preferred sync master");
+DEF_PARM_BOOL(pcm_tasklet, 0, 0644, "Handle PCM in a tasklet (lower interrupt load)");
+DEF_PARM(int, disable_pcm, 0, 0644, "Disable all PCM transmissions");
+DEF_PARM(int, print_dbg, 0, 0644, "Print DBG statements");
+DEF_PARM_BOOL(zap_autoreg, 0, 0644, "Register spans automatically (1) or not (0)");
+DEF_PARM_BOOL(prefmaster, 1, 0644, "Do we want to be zaptel preferred sync master");
 #ifdef	XPP_EC_CHUNK
-DEF_PARM_BOOL(xpp_ec, 1, 0400, "Do we use our own (1) or Zaptel's (0) echo canceller");
+DEF_PARM_BOOL(xpp_ec, 0, 0444, "Do we use our own (1) or Zaptel's (0) echo canceller");
 #else
 static int xpp_ec = 0;
 #endif
@@ -102,11 +107,46 @@ static int xpp_ec = 0;
 #include "supress/ec_xpp.h"
 #endif
 
+#ifdef	DEBUG_SYNC_PARPORT
+/*
+ * Use parallel port to sample our PCM sync and diagnose quality and
+ * potential problems. A logic analizer or a scope should be connected
+ * to the data bits of the parallel port.
+ *
+ * Array parameter: Choose the two xbuses Id's to sample.
+ *                  This can be changed on runtime as well. Example:
+ *                    echo "3,5" > /sys/module/xpp/parameters/parport_xbuses
+ */
+static int parport_xbuses[2] = { 0, 1 };
+unsigned int parport_xbuses_num_values;
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
+module_param_array(parport_xbuses, int, &parport_xbuses_num_values, 0577);
+#else
+module_param_array(parport_xbuses, int, parport_xbuses_num_values, 0577);
+#endif
+MODULE_PARM_DESC(parport_xbuses, "Id's of xbuses to sample (1-2)");
+
+/*
+ * Flip a single bit in the parallel port:
+ *   - The bit number is either bitnum0 or bitnum1
+ *   - Bit is selected by xbus number from parport_xbuses[]
+ */
+void xbus_flip_bit(xbus_t *xbus, unsigned int bitnum0, unsigned int bitnum1)
+{
+	int		num = xbus->num;
+
+	if(num == parport_xbuses[0])
+		flip_parport_bit(bitnum0);
+	if(num == parport_xbuses[1])
+		flip_parport_bit(bitnum1);
+}
+EXPORT_SYMBOL(xbus_flip_bit);
+#endif
 
 static void xpp_tick(unsigned long param);
 static int zaptel_register_xpd(xpd_t *xpd);
 static int zaptel_unregister_xpd(xpd_t *xpd);
-static void xpp_transmitprep(xpd_t *xpd, xpp_line_t lines, xpacket_t *pack);
+static void xpp_transmitprep(xbus_t *xbus, int unit, xpp_line_t lines, xpacket_t *pack);
 static void xpp_receiveprep(xpd_t *xpd);
 static int xpd_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data);
 static int proc_xpd_ztregister_read(char *page, char **start, off_t off, int count, int *eof, void *data);
@@ -129,10 +169,10 @@ static void external_sync(xbus_t *the_xbus)
 			continue;
 		if (!xbus->hardware_exists)
 			continue;
-		CALL_PROTO(GLOBAL, SYNC_SOURCE, xbus, NULL, 1, 0);
+		CALL_PROTO(GLOBAL, SYNC_SOURCE, xbus, NULL, SYNC_MODE_HOST, 0);
 	}
 	if(the_xbus)
-		CALL_PROTO(GLOBAL, SYNC_SOURCE, the_xbus, NULL, 1, 1);
+		CALL_PROTO(GLOBAL, SYNC_SOURCE, the_xbus, NULL, SYNC_MODE_AB, 0);
 }
 
 /*
@@ -173,70 +213,94 @@ static void sync_master_is(xbus_t *xbus)
 static inline void send_pcm_frame(xbus_t *xbus, xframe_t *xframe)
 {
 	static int	rate_limit;
+	unsigned long	flags;
+	struct timeval	now;
+	unsigned long	sec_diff;
+	unsigned long	usec_diff;
 
-	if(disable_pcm)
-		xbus->ops->xframe_free(xbus, xframe);
-	else {
-		if(print_dbg && ((rate_limit++ % 1003) == 0))
-			dump_xframe("SEND_PCM", xbus, xframe);
-		xframe_send(xbus, xframe);
-		XBUS_COUNTER(xbus, SEND_PCM)++;
+	spin_lock_irqsave(&xbus->lock, flags);
+	do_gettimeofday(&now);
+	if(unlikely(disable_pcm || !xbus->hardware_exists))
+		goto dropit;
+	sec_diff = now.tv_sec - xbus->last_tx_sync.tv_sec;
+	usec_diff = sec_diff * 1000000 + (now.tv_usec - xbus->last_tx_sync.tv_usec);
+	if(unlikely(abs(sec_diff) > 2)) {
+		DBG("%s: PCM TX timing restart (sec_diff=%ld)\n",
+					xbus->busname, sec_diff);
+		xbus->last_tx_sync = now;
+		goto dropit;
 	}
+	if(abs(usec_diff - 1000) > TICK_TOLERANCE) {
+		if(print_dbg && printk_ratelimit())
+			DBG("%s: Bad PCM TX timing: usec_diff=%ld.\n",
+					xbus->busname, usec_diff);
+	}
+	if(usec_diff > xbus->max_tx_sync)
+		xbus->max_tx_sync = usec_diff;
+	if(usec_diff < xbus->min_tx_sync)
+		xbus->min_tx_sync = usec_diff;
+	spin_unlock_irqrestore(&xbus->lock, flags);
+	/* OK, really send it */
+	if(print_dbg == 2 && ((rate_limit++ % 1003) == 0))
+		dump_xframe("SEND_PCM", xbus, xframe);
+	xframe_send(xbus, xframe);
+	XBUS_COUNTER(xbus, SEND_PCM)++;
+	xbus->last_tx_sync = now;
+	return;
+dropit:
+	spin_unlock_irqrestore(&xbus->lock, flags);
+	xbus->ops->xframe_free(xbus, xframe);
 }
 
 /*
  * Calculate PCM line_count and lines from a xbus+unit number:
  *   - Takes into account BRI subunits
  *   - Returns an XPD pointer if we should transmit, NULL otherwise
+ *   - Calls zt_transmit() for all registered subunits so we empty
+ *     the relevant zaptel buffers (otherwise asterisk fails to write)
+ *     and we have stable buffers to send.
  */
 static xpd_t *unit_pcm_calc(const xbus_t *xbus, int unit, size_t *pcm_len, xpp_line_t *plines)
 {
-	bool		is_bri;
-	bool		digital_telephony;
 	int		line_count = 0;
 	xpp_line_t	lines;
-	xpd_t		*xpd;
+	xpd_t		*xpd = NULL;
+	xpd_t		*tmp_xpd;
 	unsigned long	flags;
 	int		i;
+	int		subunit;
 
-	xpd = xpd_by_addr(xbus, unit, 0);
-	if(!xpd || !xpd->card_present)
-		return NULL;
-	spin_lock_irqsave(&xpd->lock, flags);
-	is_bri = (xpd->type == XPD_TYPE_BRI_NT) || (xpd->type == XPD_TYPE_BRI_TE);
-	digital_telephony = is_bri;	/* or PRI */
-	if(digital_telephony) {
-		int		subunit;
+	lines = 0;
+	line_count = 0;
+	/*
+	 * BRI has a single PCM highway for all subunits, so
+	 * we agregate the next subunits into the same
+	 * transmition. We shift 4 bits (B1, B2, D, E)
+	 */
+	for(subunit = 0; subunit < MAX_SUBUNIT; subunit++) {
+		xpp_line_t	tmp_lines;
 
-		/* without D-Channel */
-		lines = xpd->offhook & ~xpd->digital_signalling;
-		for_each_line(xpd, i)
-			if(IS_SET(lines, i))
-				line_count++;
-		if(is_bri) {
+		tmp_xpd = xpd_by_addr(xbus, unit, subunit);
+		if(!tmp_xpd || !tmp_xpd->card_present)
+			continue;
+		if(!xpd)
+			xpd = tmp_xpd;	/* First xpd for our unit */
+		if(SPAN_REGISTERED(tmp_xpd)) {
 			/*
-			 * BRI has a single PCM highway for all subunits, so
-			 * we agregate the next subunits into the same
-			 * transmition.
+			 * calls to zt_transmit should be out of spinlocks, as it may call back
+			 * our hook setting methods.
 			 */
-			for(subunit = 1; subunit < MAX_SUBUNIT; subunit++) {
-				xpd_t		*tmp_xpd = xpd_by_addr(xbus, unit, subunit);
-				xpp_line_t	tmp_lines;
-
-				if(!tmp_xpd || !tmp_xpd->card_present)
-					continue;
-				tmp_lines = xpd->offhook & ~xpd->digital_signalling; /* Without D-channel */
-				for_each_line(tmp_xpd, i)
-					if(IS_SET(tmp_lines, i))
-						line_count++;
-				lines = (lines << SUBUNIT_PCM_SHIFT) | tmp_lines;	/* B1, B2, D, E */
-			}
+			zt_transmit(&tmp_xpd->span);
 		}
-	} else {
-		lines = (xpd->offhook | xpd->cid_on);
-		for_each_line(xpd, i)
-			if(IS_SET(lines, i))
+		spin_lock_irqsave(&tmp_xpd->lock, flags);
+		tmp_lines = (tmp_xpd->offhook | tmp_xpd->cid_on) & ~tmp_xpd->digital_signalling;	/* without D-Channel */
+		lines |= (tmp_lines << (SUBUNIT_PCM_SHIFT * subunit));
+		for_each_line(tmp_xpd, i)
+			if(IS_SET(tmp_lines, i))
 				line_count++;
+		spin_unlock_irqrestore(&tmp_xpd->lock, flags);
+		if(!IS_BRI(tmp_xpd))
+			break;	/* no subunits */
 	}
 	/*
 	 * FIXME: Workaround a bug in sync code of the Astribank.
@@ -246,9 +310,10 @@ static xpd_t *unit_pcm_calc(const xbus_t *xbus, int unit, size_t *pcm_len, xpp_l
 		lines = BIT(0);
 		line_count = 1;
 	}
+	if(!line_count)
+		return NULL;	/* Nothing to transmit for this unit */
 	*pcm_len = RPACKET_HEADERSIZE + sizeof(xpp_line_t)  +  line_count * ZT_CHUNKSIZE;
 	*plines = lines;
-	spin_unlock_irqrestore(&xpd->lock, flags);
 	return xpd;
 }
 
@@ -285,18 +350,15 @@ static void xbus_tick(xbus_t *xbus)
 			xpd_set_addr(&pack->addr, xpd->id);
 			pack->datalen = pcm_len;
 			RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, lines) = lines;
-			xpp_transmitprep(xpd, lines, pack);
+			xpp_transmitprep(xbus, unit, lines, pack);
 		}
-
 	}
 	if(xframe)	/* clean any leftovers */
 		send_pcm_frame(xbus, xframe);
-
 	for(i = 0; i < MAX_XPDS; i++) {
 		xpd = xpd_of(xbus, i);
 		if(!xpd || !xpd->card_present)
 			continue;
-
 		if(SPAN_REGISTERED(xpd))
 			xpp_receiveprep(xpd);
 		xpd->timer_count++;
@@ -311,10 +373,11 @@ static void xbus_tick(xbus_t *xbus)
 void xpp_tick(unsigned long param)
 {
 	xbus_t		*xbus;
-	int		i, j;
+	int		i;
 
 #ifdef	ZAPTEL_SYNC_TICK
-	do_gettimeofday (&ticked_xpp);
+	flip_parport_bit(3);
+	do_gettimeofday(&ticked_xpp);
 #endif
 	if(!sync_master)			/* Called from timer */
 		mod_timer(&xpp_timer, jiffies + 1);	/* Must be 1KHz rate */
@@ -335,26 +398,30 @@ void xpp_tick(unsigned long param)
 			DBG("Dropped packet. %s is in_use\n", xbus->busname);
 			continue;
 		}
-		/*
-		 * calls to zt_transmit should be out of spinlocks, as it may call back
-		 * our hook setting methods.
-		 */
-		for(j = 0; j < MAX_XPDS; j++) {
-			xpd_t	*xpd = xpd_of(xbus, j);
-
-			if(xpd && SPAN_REGISTERED(xpd)) {
-				zt_transmit(&xpd->span);
-			}
-		}
 		xbus_tick(xbus);
+		if((xpp_timer_count % (60*1000)) == 0) {
+			xbus->min_tx_sync = INT_MAX;
+			xbus->max_tx_sync = 0;
+			xbus->min_rx_sync = INT_MAX;
+			xbus->max_rx_sync = 0;
+		}
 		up_read(&xbus->in_use);
 	}
 }
 
-void got_pcm_from(xpd_t *xpd)
+void got_sync_from(xpd_t *xpd)
 {
-	if(xpd != sync_master)
+	xbus_t		*xbus;
+
+	xbus = xpd->xbus;
+	if(xpd != sync_master) {
+		static int rate_limit;
+
+		if((rate_limit++ % 1003) == 0)
+			DBG("%s/%s: is not SYNC master. Go away! (%d)\n",
+					xbus->busname, xpd->xpdname, rate_limit);
 		return;
+	}
 	atomic_inc(&missed_ticks);
 	if(!pcm_tasklet) {
 		xpp_tick(0L);
@@ -364,46 +431,92 @@ void got_pcm_from(xpd_t *xpd)
 }
 
 #ifdef	ZAPTEL_SYNC_TICK
-static void sync_status_clear(void)
+static void send_drift(int drift)
 {
-	struct timeval		now;
+	struct timeval          now;
+	const char              *msg;
+	xbus_t			*xbus;
 
-	do_gettimeofday (&now);
-	usec_sigma = 0;
-	status_cleared_at = now.tv_sec;
+	BUG_ON(!sync_master);
+	xbus = sync_master->xbus;
+	BUG_ON(drift < SYNC_ADJ_MIN || drift > SYNC_ADJ_MAX);
+	do_gettimeofday(&now);
+	if(drift == 0)
+		msg = "stop";
+	else if(drift > 0)
+		msg = "up";
+	else
+		msg = "down";
+	DBG("DRIFT adjust %s (%d) (last update %ld seconds ago)\n",
+		msg, drift, now.tv_sec - xbus->pll_updated_at);
+	CALL_PROTO(GLOBAL, SYNC_SOURCE, xbus, NULL, SYNC_MODE_PLL, drift);
+	xbus->pll_updated_at = now.tv_sec;
 }
 
 int zaptel_sync_tick(struct zt_span *span, int is_master)
 {
 	xpd_t		*xpd = span->pvt;
+	int		offset;
 
-	if(!sync_tick_active) {
-		zaptel_is_ticking = 0;
-		usec_sigma = 0;
-		return 0;	/* No auto sync from zaptel */
+	if(!sync_tick_active)
+		goto noop;
+	/*
+	 * Detect if any of our spans is zaptel sync master
+	 */
+	if(is_master) {
+		sync_tick_nomaster = 0;		/* Yes */
+		have_sync_mastership = 1;
 	}
+	if(sync_tick_nomaster < MAX_BUSES * MAX_XPDS) {
+		sync_tick_nomaster++;	/* Maybe */
+		goto noop;
+	}
+	/* Now we know for sure someone else is zaptel sync master */
+	have_sync_mastership = 0;
 	BUG_ON(!xpd);
 	/*
 	 * Calculate only if:
-	 *   - Called for the sync_master
 	 *   - HOST sync (for information only)
+	 *   - Called for the sync_master (the function is called
+	 *     for each span, so we don't want to do it multiple times).
 	 */
-	if(sync_master && xpd != sync_master)
+	if(sync_master && sync_master != xpd)
 		return 0;
-	do_gettimeofday (&ticked_zaptel);
+	zaptel_tick_count++;
+	zaptel_is_ticking = 1;
+	flip_parport_bit(1);
+	do_gettimeofday(&ticked_zaptel);
 	usec_lag_curr =	
 		(ticked_zaptel.tv_sec - ticked_xpp.tv_sec)*1000*1000 +
 		(ticked_zaptel.tv_usec - ticked_xpp.tv_usec);
-	if(usec_lag_curr > ZAPTEL_BIG_LAG) {
-		if(printk_ratelimit())
-			NOTICE("Big lag of Xpp ticks relative to zaptel ticks: %ld\n", usec_lag_curr);
+	if(unlikely(usec_lag_curr > ZAPTEL_BIG_LAG)) {
+		static int	rate_limit;
+
+		if((rate_limit++ % 1003) == 0)
+			DBG("Big lag of Xpp ticks relative to zaptel ticks: %ld\n", usec_lag_curr);
+	}
+	sigma_lag += usec_lag_curr;
+	if((zaptel_tick_count % SYNC_ADJ_INTERVAL) == 0) {
+		average_lag = sigma_lag / SYNC_ADJ_INTERVAL;
+		sigma_lag = 0;
+		if(sync_master == xpd) {
+			xbus_t	*xbus = sync_master->xbus;
+
+			/* Calculate required fix */
+			offset = (average_lag - 500) / SYNC_ADJ_FACTOR;
+			if(offset < SYNC_ADJ_MIN)
+				offset = SYNC_ADJ_MIN;
+			if(offset > SYNC_ADJ_MAX)
+				offset = SYNC_ADJ_MAX;
+			if(xbus->sync_adjustment != offset)	/* Only fix if needed */
+				send_drift(offset);
+		}
 	}
-	usec_delta = usec_lag_curr - usec_lag_prev;
-	usec_sigma += usec_delta;
-	zaptel_is_ticking = 1;
-	since_status_clear = ticked_zaptel.tv_sec - status_cleared_at;
-	usec_lag_prev = usec_lag_curr;
 	return 0;
+noop:
+	zaptel_is_ticking = 0;
+	sigma_lag = 0;
+	return 0;	/* No auto sync from zaptel */
 }
 #endif
 
@@ -458,7 +571,7 @@ static void xpd_free(xpd_t *xpd)
 
 #define	REV(x,y)	(10 * (x) + (y))
 static byte good_revs[] = {
-	REV(2,4),
+	REV(2,6),
 };
 #undef	REV
 
@@ -578,11 +691,14 @@ void card_detected(struct card_desc_struct *card_desc)
 	xbus_register_xpd(xbus, xpd);
 	if(CALL_XMETHOD(card_init, xbus, xpd) < 0)
 		goto err;
-	// Turn off all channels
-	CALL_XMETHOD(XPD_STATE, xbus, xpd, 0);
+	CALL_XMETHOD(XPD_STATE, xbus, xpd, 0);				/* Turn off all channels */
+	CALL_PROTO(GLOBAL, SYNC_SOURCE, xbus, NULL, SYNC_MODE_HOST, 0);	/* Not a SYNC master (yet) */
 	xpd->card_present = 1;
-	// Turn on all channels
-	CALL_XMETHOD(XPD_STATE, xbus, xpd, 1);
+	if(xbus->last_tx_sync.tv_sec == 0 && xbus->last_tx_sync.tv_usec == 0) {
+		do_gettimeofday(&xbus->last_tx_sync);			/* start timing ticks */
+		xbus->last_rx_sync = xbus->last_tx_sync;
+	}
+	CALL_XMETHOD(XPD_STATE, xbus, xpd, 1);				/* Turn on all channels */
 
 	if(zap_autoreg)
 		zaptel_register_xpd(xpd);
@@ -909,40 +1025,48 @@ int proc_sync_read(char *page, char **start, off_t off, int count, int *eof, voi
 {
 	int		len = 0;
 	unsigned int	xpp_timer_rate;
-	unsigned int	now;
+	unsigned int	jiffies_now;
 
 	len += sprintf(page + len, "# To modify sync source write into this file:\n");
 	len += sprintf(page + len, "#     HOST        - For host based sync\n");
 	len += sprintf(page + len, "#     SYNC=nn     - XBUS-nn provide sync\n");
-#ifdef	ZAPTEL_SYNC_TICK
-	len += sprintf(page + len, "#     CLS         - Clear Statistics\n");
-#endif
 	len += sprintf(page + len, "#     QUERY=nn    - Query XBUS-nn for sync information (DEBUG)\n");
 	if(!sync_master)
 		len += sprintf(page + len, "HOST\n");
-	else
+	else {
 		len += sprintf(page + len, "SYNC=%02d\n", sync_master->xbus->num);
+		len += sprintf(page + len, "PLL DRIFT=%d\n",
+				sync_master->xbus->sync_adjustment);
+	}
 #ifdef	ZAPTEL_SYNC_TICK
 	if(sync_tick_active) {
-		len += sprintf(page + len, "\nZaptel Reference Sync:\n");
+		len += sprintf(page + len, "\nZaptel Reference Sync (%s):\n",
+				(have_sync_mastership)?"xpp":"external");
 		if(zaptel_is_ticking) {
+			if(sync_master) {
+				struct timeval	now;
+				xbus_t		*xbus = sync_master->xbus;
+
+				do_gettimeofday(&now);
+				len += sprintf(page + len, "\t%-19s: %5ld seconds ago\n",
+						"PLL updated",
+						(xbus->pll_updated_at == 0) ? 0 : now.tv_sec - xbus->pll_updated_at);
+			}
 			len += sprintf(page + len, "\t%-19s: %5ld (usec)\n",
 					"current lag", usec_lag_curr);
 			len += sprintf(page + len, "\t%-19s: %5ld (usec)\n",
-					"delta lag", usec_delta);
-			len += sprintf(page + len, "\t%-19s: %5ld (usec)\n",
-					"sigma of deltas", usec_sigma);
-			len += sprintf(page + len, "\t%-19s: %5ld seconds ago\n",
-					"status cleared", since_status_clear);
+					"average lag", average_lag);
 		} else
 			len += sprintf(page + len, "\tNot activated\n");
 	}
+	len += sprintf(page + len, "zaptel_tick: #%d\n", zaptel_tick_count);
+	len += sprintf(page + len, "tick - zaptel_tick = %d\n", xpp_timer_count - zaptel_tick_count);
 #endif
 	len += sprintf(page + len, "\ntick: #%d\n", xpp_timer_count);
 	xpp_timer_rate = 0;
-	now = jiffies;
-	if(now - xpp_last_jiffies > 0) {
-		unsigned long delta = (now - xpp_last_jiffies);
+	jiffies_now = jiffies;
+	if(jiffies_now - xpp_last_jiffies > 0) {
+		unsigned long delta = (jiffies_now - xpp_last_jiffies);
 		xpp_timer_rate = (xpp_timer_count % SAMPLE_TICKS) * HZ / delta;
 		len += sprintf(page + len, "tick rate: %4d/second (SAMPLE_TICKS=%d)\n", xpp_timer_rate, SAMPLE_TICKS);
 	}
@@ -974,10 +1098,6 @@ static int proc_sync_write(struct file *file, const char __user *buffer, unsigne
 	buf[count] = '\0';
 	if(strncmp("HOST", buf, 4) == 0) {
 		sync_master_is(NULL);
-#ifdef	ZAPTEL_SYNC_TICK
-	} else if(strncmp("CLS", buf, 3) == 0) {
-		sync_status_clear();
-#endif
 	} else if(sscanf(buf, "SYNC=%d", &xbus_num) == 1) {
 		DBG("SYNC=%d\n", xbus_num);
 		if((xbus = xbus_of(xbus_num)) == NULL) {
@@ -991,7 +1111,7 @@ static int proc_sync_write(struct file *file, const char __user *buffer, unsigne
 			ERR("No bus %d exists\n", xbus_num);
 			return -ENXIO;
 		}
-		CALL_PROTO(GLOBAL, SYNC_SOURCE, xbus, NULL, 0, 0);
+		CALL_PROTO(GLOBAL, SYNC_SOURCE, xbus, NULL, SYNC_MODE_QUERY, 0);
 	} else if(sscanf(buf, "%d %d", &xbus_num, &xpd_num) == 2) {
 		NOTICE("Using deprecated syntax to update /proc/%s/%s\n", 
 				PROC_DIR, PROC_SYNC);
@@ -1149,29 +1269,29 @@ error:
 
 #define	PREP_REPORT_RATE	1000
 
-static void xpp_transmitprep(xpd_t *xpd, xpp_line_t lines, xpacket_t *pack)
+static void xpp_transmitprep(xbus_t *xbus, int unit, xpp_line_t lines, xpacket_t *pack)
 {
 	byte		*pcm;
-	int		channels;
 	struct zt_chan	*chans;
 	unsigned long	flags;
 	int		i;
 	int		subunit;
-	xpd_t		*tmp_xpd;
 
 
-	BUG_ON(!xpd);
+	BUG_ON(!xbus);
 	BUG_ON(!pack);
 	pcm = RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, pcm);
-	for(subunit = 0; subunit < MAX_SUBUNIT; subunit++) {
-		tmp_xpd = xpd_by_addr(xpd->xbus, xpd->addr.unit, subunit);
-		if(!tmp_xpd)
+	for(subunit = 0; subunit < MAX_SUBUNIT; subunit++, lines >>= SUBUNIT_PCM_SHIFT) {
+		xpd_t		*tmp_xpd;
+
+		tmp_xpd = xpd_by_addr(xbus, unit, subunit);
+		if(!tmp_xpd || !tmp_xpd->card_present)
 			continue;
+		if(lines == 0)
+			break;	/* Optimize */
 		spin_lock_irqsave(&tmp_xpd->lock, flags);
-	
-		channels = tmp_xpd->channels;
 		chans = tmp_xpd->span.chans;
-		for (i = 0; i < channels; i++) {
+		for (i = 0; i < tmp_xpd->channels; i++) {
 			if(IS_SET(lines, i)) {
 				if(SPAN_REGISTERED(tmp_xpd)) {
 					memcpy((u_char *)pcm, chans[i].writechunk, ZT_CHUNKSIZE);
@@ -1184,7 +1304,8 @@ static void xpp_transmitprep(xpd_t *xpd, xpp_line_t lines, xpacket_t *pack)
 		}
 		XPD_COUNTER(tmp_xpd, PCM_WRITE)++;
 		spin_unlock_irqrestore(&tmp_xpd->lock, flags);
-		lines >>= SUBUNIT_PCM_SHIFT;
+		if(!IS_BRI(tmp_xpd))
+			break; /* only BRI has subunits */
 	}
 }
 
@@ -1245,10 +1366,8 @@ static void xpp_receiveprep(xpd_t *xpd)
 	int	channels = xpd->channels;
 	struct zt_chan	*chans = xpd->span.chans;
 	unsigned long	flags;
-	bool	digital_telephony;
 
 	spin_lock_irqsave(&xpd->lock, flags);
-	digital_telephony = (xpd->type == XPD_TYPE_BRI_NT) || (xpd->type == XPD_TYPE_BRI_TE);
 //	if((xpd->timer_count % PREP_REPORT_RATE) == 0)
 //		DBG("%d\n", xpd->timer_count);
 
@@ -1266,7 +1385,7 @@ static void xpp_receiveprep(xpd_t *xpd)
 		 */
 		if(IS_SET(xpd->digital_signalling, i))
 			continue;
-		if(IS_SET(xpd->offhook, i) || IS_SET(xpd->cid_on, i) || digital_telephony) {
+		if(IS_SET(xpd->offhook, i) || IS_SET(xpd->cid_on, i) || IS_BRI(xpd)) {
 			// memset((u_char *)readchunk, 0x5A, ZT_CHUNKSIZE);	// DEBUG
 			// fill_beep((u_char *)readchunk, 1, 1);	// DEBUG: BEEP
 			memcpy(chans[i].readchunk, (u_char *)readchunk, ZT_CHUNKSIZE);
@@ -1274,7 +1393,6 @@ static void xpp_receiveprep(xpd_t *xpd)
 			memset(chans[i].readchunk, 0x7F, ZT_CHUNKSIZE);	// SILENCE
 		}
 	}
-
 #if WITH_ECHO_SUPPRESSION
 	/* FIXME: need to Echo cancel double buffered data */
 	for (i = 0;i < xpd->span.channels; i++) {
@@ -1412,8 +1530,9 @@ int xpp_ioctl(struct zt_chan *chan, unsigned int cmd, unsigned long arg)
 		case ZT_TONEDETECT:
 			if (get_user(x, (int __user *)arg))
 				return -EFAULT;
-			DBG("xpd=%d: ZT_TONEDETECT chan=%d: TONEDETECT_ON=%d TONEDETECT_MUTE=%d\n",
-				xpd->id, pos, (x & ZT_TONEDETECT_ON), (x & ZT_TONEDETECT_MUTE));
+			DBG("%s/%s/%d: ZT_TONEDETECT (TONEDETECT_ON=%d TONEDETECT_MUTE=%d)\n",
+				xpd->xbus->busname, xpd->xpdname, pos,
+				(x & ZT_TONEDETECT_ON), (x & ZT_TONEDETECT_MUTE));
 			return -ENOTTY;
 		default:
 			/* Some span-specific commands before we give up: */
@@ -1747,7 +1866,6 @@ int __init xpp_zap_init(void)
 #else
 	INFO("FEATURE: %s without sync_tick() from ZAPTEL\n", THIS_MODULE->name);
 #endif
-
 #ifdef CONFIG_PROC_FS
 	xpp_proc_toplevel = proc_mkdir(PROC_DIR, NULL);
 	if(!xpp_proc_toplevel) {
@@ -1773,9 +1891,6 @@ int __init xpp_zap_init(void)
 	/* Only timer init. We add it only *after* zt_register */
 	init_timer(&xpp_timer);
 	sync_master_is(NULL);			/* Internal ticking */
-#ifdef	ZAPTEL_SYNC_TICK
-	sync_status_clear();
-#endif
 	return 0;
 err:
 	do_cleanup();
