@@ -34,7 +34,7 @@ void update_xpd_status(xpd_t *xpd, int alarm_flag);
 void update_zap_ring(xpd_t *xpd, int pos, bool on);
 void update_line_status(xpd_t *xpd, int pos, bool good);
 void fill_beep(u_char *buf, int num, int duration);
-void got_pcm_from(xpd_t *xpd);
+void got_sync_from(xpd_t *xpd);
 int xpp_open(struct zt_chan *chan);
 int xpp_close(struct zt_chan *chan);
 int xpp_ioctl(struct zt_chan *chan, unsigned int cmd, unsigned long arg);
