@@ -28,7 +28,7 @@
 #include <linux/module.h>
 #include <linux/delay.h>
 
-static const char rcsid[] = "$Id: xproto.c 2239 2007-02-27 06:14:18Z tzafrir $";
+static const char rcsid[] = "$Id: xproto.c 2475 2007-04-29 20:19:13Z tzafrir $";
 
 extern	int print_dbg;
 
@@ -235,7 +235,9 @@ int xframe_receive(xbus_t *xbus, xframe_t *xframe)
 			ret = -EPROTO;
 			goto out;
 		}
-		packet_process(xbus, pack);
+		ret = packet_process(xbus, pack);
+		if(unlikely(ret < 0))
+			goto out;
 	} while(p < xframe_end);
 out:
 	xbus->ops->xframe_free(xbus, xframe);
