@@ -919,6 +919,9 @@ struct zt_tone_def {		/* Structure for zone programming */
 #define	ZT_CONF_CONFANNMON 7		/* conference announce/monitor mode */
 #define	ZT_CONF_REALANDPSEUDO 8	/* real and pseudo port both on conf */
 #define ZT_CONF_DIGITALMON 9	/* Do not decode or interpret */
+#define	ZT_CONF_MONITOR_RX_PREECHO 10	/* monitor mode (rx of other chan) - before echo can is done */
+#define	ZT_CONF_MONITOR_TX_PREECHO 11	/* monitor mode (tx of other chan) - before echo can is done */
+#define	ZT_CONF_MONITORBOTH_PREECHO 12	/* monitor mode (rx & tx of other chan) - before echo can is done */
 #define	ZT_CONF_FLAG_MASK 0xff00	/* mask for flags */
 #define	ZT_CONF_LISTENER 0x100		/* is a listener on the conference */
 #define	ZT_CONF_TALKER 0x200		/* is a talker on the conference */
@@ -1069,6 +1072,7 @@ struct zt_chan {
 	u_char swritechunk[ZT_MAX_CHUNKSIZE];	/* Buffer to be written */
 	u_char *readchunk;						/* Actual place to read from */
 	u_char sreadchunk[ZT_MAX_CHUNKSIZE];	/* Preallocated static area */
+	short *readchunkpreec;
 	
 	/* Pointer to tx and rx gain tables */
 	u_char *rxgain;
