@@ -549,5 +549,23 @@ struct tone_zone builtin_zones[]  =
             { ZT_TONE_STUTTER, "!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,425" },
           },
         },
+        { 39, "ph", "Philippines", { 1000, 4000 },
+          {
+            /* Reference: http://www.itu.int/ITU-T/inr/forms/files/tones-0203.pdf */
+            { ZT_TONE_DIALTONE, "425" },
+            { ZT_TONE_BUSY, "480+620/500,0/500" },
+            { ZT_TONE_RINGTONE, "425+480/1000,0/4000" },
+            { ZT_TONE_CONGESTION, "480+620/250,0/250" },
+            { ZT_TONE_CALLWAIT, "440/300,0/10000" },
+            /* DIAL RECALL - not specified */
+            { ZT_TONE_DIALRECALL, "!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,350+440" },
+            /* RECORD TONE - not specified */
+            { ZT_TONE_RECORDTONE, "1400/500,0/15000" },
+            /* INFO TONE - not specified */
+            { ZT_TONE_INFO, "!950/330,!1400/330,!1800/330,0" },
+            /* STUTTER TONE - not specified */
+            { ZT_TONE_STUTTER, "!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,350+440" },
+          },
+        },
 	{ -1 }
 };
